package de.qwerty287.ftpclient.ui.files

import de.qwerty287.ftpclient.R
import de.qwerty287.ftpclient.providers.File

object FileExtensions {
    private val audioFormats = listOf(
        "aac",
        "adts",
        "ass",
        "mka",
        "minipsf",
        "it",
        "oga",
        "ogg",
        "opus",
        "xm",
        "s3m",
        "axa",
        "psf",
        "pls",
        "mo3",
        "sid",
        "psid",
        "aax",
        "psflib",
        "spx",
        "mpc",
        "mpp",
        "mp+",
        "opus",
        "wvc",
        "m4b",
        "f4b",
        "oga",
        "ogg",
        "spx",
        "xi",
        "aifc",
        "aiffc",
        "mp3",
        "mpga",
        "au",
        "snd",
        "wma",
        "gsm",
        "wv",
        "wvp",
        "m3u",
        "m3u8",
        "vlc",
        "awb",
        "ape",
        "dts",
        "oga",
        "ogg",
        "voc",
        "mod",
        "ult",
        "uni",
        "m15",
        "mtm",
        "669",
        "med",
        "mid",
        "midi",
        "kar",
        "asx",
        "wax",
        "wvx",
        "wmx",
        "wav",
        "flac",
        "ac3",
        "mp2",
        "aiff",
        "aif",
        "dtshd",
        "loas",
        "xhe",
        "aa",
        "m4a",
        "f4a",
        "xmf",
        "amr",
        "m4r",
        "amz",
        "stm",
        "pla",
        "tta",
        "ra",
        "rax",
        "oga",
        "ogg",
        // extensions below this comment are extracted from application mime types
        "shn",
        "xspf",
        "mmf",
        "smaf"
    )
    private val videoFormats = listOf(
        "viv",
        "vivo",
        "mjpeg",
        "mjpg",
        "mkv",
        "ogv",
        "ogg",
        "fxm",
        "axv",
        "mlt",
        "westley",
        "dv",
        "wmp",
        "webm",
        "nsv",
        "ogm",
        "mpeg",
        "mpg",
        "mp2",
        "mpe",
        "vob",
        "vdr",
        "flv",
        "qt",
        "mov",
        "moov",
        "qtvr",
        "3gp",
        "3gpp",
        "3ga",
        "3g2",
        "3gp2",
        "3gpp2",
        "avi",
        "avf",
        "divx",
        "mj2",
        "mjp2",
        "ogg",
        "movie",
        "rv",
        "rvx",
        "m1u",
        "m4u",
        "mxu",
        "fli",
        "flc",
        "mp4",
        "m4v",
        "f4v",
        "lrv",
        "m2t",
        "m2ts",
        "ts",
        "mts",
        "cpi",
        "clpi",
        "mpl",
        "mpls",
        "bdm",
        "bdmv",
        "wmv",
        "mk3d",
        "mng",
        // extensions below this comment are extracted from application mime types
        "nsc",
        "sdp",
        "ogx",
        "anx",
        "swf",
        "spl",
        "mxf",
        "wpl",
        "smil",
        "smi",
        "sml",
        "kino",
        "qtl",
        "rm",
        "rmj",
        "rmm",
        "rms",
        "rmx",
        "rmvb",
        "yt"
    )
    private val previewableImageFormats = listOf(
        "bmp",
        "gif",
        "jpg",
        "jpeg",
        "jpe",
        "png",
        "webp",
        "heic",
        "heif",
    )
    private val imageFormats = listOf(
        "pic",
        "psd",
        "xcf",
        "fig",
        "bay",
        "bmq",
        "cs1",
        "cs2",
        "erf",
        "fff",
        "hrd",
        "mdc",
        "mos",
        "pnx",
        "rdc",
        "ief",
        "dib",
        "nef",
        "emf",
        "orf",
        "wmf",
        "lws",
        "sgi",
        "avifs",
        "ppm",
        "k25",
        "xwd",
        "rgb",
        "j2c",
        "j2k",
        "jpc",
        "hdr",
        "pic",
        "dxf",
        "xbm",
        "cgm",
        "ag",
        "pat",
        "rle",
        "svgz",
        "ora",
        "tif",
        "tiff",
        "ico",
        "jpm",
        "jpgm",
        "ktx",
        "exr",
        "dcr",
        "rp",
        "dwg",
        "pcd",
        "dng",
        "gbr",
        "mrw",
        "sr2",
        "crw",
        "eps",
        "epsi",
        "epsf",
        "jng",
        "gih",
        "fits",
        "lwo",
        "lwob",
        "rw2",
        "srf",
        "iff",
        "ilbm",
        "lbm",
        "3ds",
        "pef",
        "jpf",
        "jpx",
        "raf",
        "dds",
        "mdi",
        "pgm",
        "djvu",
        "djv",
        "sun",
        "cur",
        "avif",
        "g3",
        "raw",
        "xpm",
        "svg",
        "wbmp",
        "pct",
        "pict",
        "pict1",
        "pict2",
        "qtif",
        "qif",
        "kdc",
        "msod",
        "cr2",
        "x3f",
        "sk",
        "sk1",
        "ras",
        "pbm",
        "tga",
        "icb",
        "tpic",
        "vda",
        "vst",
        "pntg",
        "jp2",
        "jpg2",
        "pcx",
        "djvu",
        "djv",
        "icns",
        "pnm",
        "arw",
        // extensions below this comment are extracted from application mime types
        "otg",
        "otg",
        "karbon",
        "fodg",
        "fodg",
        "kpm",
        "vsdm",
        "shape",
        "vssx",
        "odg",
        "odg",
        "vstm",
        "dia",
        "dcm",
        "std",
        "std",
        "sxd",
        "sxd",
        "cdr",
        "ai",
        "kil",
        "kon",
        "egon",
        "vstx",
        "pcl",
        "odi",
        "blender",
        "blend",
        "sda",
        "vsdx",
        "hpgl",
        "wpg",
        "vssm",
        "ufraw"
    )
    private val fontFormats = listOf(
        "woff2", "ttc", "ttf", "otf", "woff",
        // extensions below this comment are extracted from application mime types
        "afm", "gf", "ttx", "spd", "bdf", "psf", "pcf", "pk", "pfa", "pfb", "gsf"
    )
    private val spreadsheetFormats = listOf(
        // extensions below this comment are extracted from application mime types
        "siag",
        "pys",
        "qif",
        "otc",
        "ksp",
        "xls",
        "xlc",
        "xll",
        "xlm",
        "xlw",
        "xla",
        "xlt",
        "xld",
        "xls",
        "xltm",
        "xltm",
        "chrt",
        "xlam",
        "pln",
        "ods",
        "ods",
        "xltx",
        "xltx",
        "sds",
        "xlsm",
        "xlsm",
        "stc",
        "stc",
        "ots",
        "ots",
        "gnumeric",
        "pysu",
        "fods",
        "fods",
        "odc",
        "xlsb",
        "xlsb",
        "sdc",
        "xlsx",
        "xlsx",
        "123",
        "wk1",
        "wk3",
        "wk4",
        "wks",
        "oleo",
        "gnucash",
        "gnc",
        "xac",
        "as",
        "wb1",
        "wb2",
        "wb3",
        "sxc",
        "sxc"
    )
    private val locationFormats = listOf(
        // extensions below this comment are extracted from application mime types
        "shp", "osm", "osc", "gpx"
    )
    private val plainTextFormats = listOf(
        // extensions below this comment are extracted from application mime types
        "desktop",
        "txt",
        "gnuplot",
        "xhtml",
        "yaml",
        "yml",
        "atom",
        "rss",
        "xml",
        "html",
        "htm",
        "php",
        "php3",
        "php4",
        "php5",
        "phps",
        "json",
        "rb",
        "perl",
        "xsl",
        "xslt",
        "xlf",
        "xliff",
        "coffee",
        "json-patch",
        "pl",
        "PL",
        "ui",
        "out",
        "jsonld",
        "sh",
        "es",
        "xul",
        "dbk",
        "docbook",
        "asc",
        "sig",
        "pgp",
        "gpg",
        "c",
        "cpp",
        "go",
        "py",
        "java",
        "kt",
        "rs",
        "md",
        "org",
        "log",
    )
    private val documentFormats = listOf(
        // extensions below this comment are extracted from application mime types
        "odt",
        "docx",
        "doc",
        "pdf",
        "ott",
        "otf",
        "odb",
        "odf",
        "dotx",
        "dotx",
        "o",
        "xlr",
        "stw",
        "sxm",
        "abw",
        "zabw",
        "cbt",
        "cbz",
        "dot",
        "mobi",
        "prc",
        "fodt",
        "dotm",
        "oda",
        "sdw",
        "vor",
        "sgl",
        "vsd",
        "vst",
        "vsw",
        "vss",
        "wp",
        "wp4",
        "wp5",
        "wp6",
        "wpd",
        "wpp",
        "fl",
        "cb7",
        "fm",
        "kfo",
        "ez",
        "wwf",
        "rtf",
        "kud",
        "sxg",
        "wcm",
        "wdb",
        "wks",
        "wps",
        "sam",
        "sxw",
        "wri",
        "nb",
        "jpr",
        "jpx",
        "lyx",
        "glade",
        "pdb",
        "pdc",
        "epub",
        "p65",
        "pm",
        "pm6",
        "pmd",
        "cwk",
        "smf",
        "oxt",
        "aw",
        "ged",
        "gedcom",
        "dbf",
        "dvi",
        "odm",
        "pw",
        "obj",
        "lwp",
        "ps",
        "chm",
        "cdf",
        "nc",
        "gra",
        "docm",
        "hdf",
        "hdf4",
        "h4",
        "hdf5",
        "h5",
        "mdb",
        "hwt",
        "qti",
        "gp",
        "gplt",
        "602",
        "ipynb",
        "oxps",
        "xps",
        "kwd",
        "kwt",
        "cbr",
        "psw",
        "hwp",
        "flw",
        "sgf",
        "skr",
        "pkr",
        "asc",
        "key",
        "it87",
        "kdelnk",
        "srt",
        "der",
        "crt",
        "cert",
        "pem",
        "p10",
        "p10",
        "csr",
        "src",
        "smi",
        "sami",
        "mbox",
        "p7c",
        "p7m",
        "spc",
        "p7b",
        "man",
        "p7s",
        "p7s",
        "toc",
        "rnc",
        "pgn",
        "cue",
        "ccmx",
        "ica",
        "pcf",
        "dtd",
        "ent",
        "oth",
        "xht",
        "xbel",
        "xbl",
        "xsd",
        "rng",
        "js",
        "jsm",
        "mjs",
        "jrd",
        "jnlp",
        "la",
        "asp",
        "csh",
        "m4",
        "mab",
        "awk",
        "siv",
        "pm",
        "al",
        "pod",
        "kcfgc",
        "notifyrc",
        "kcfg",
        "rc",
    )
    private val packageFormats = listOf(
        // extensions below this comment are extracted from application mime types
        "shar",
        "flatpakrepo",
        "emp",
        "tlrz",
        "spm",
        "zz",
        "flatpakref",
        "sv4cpio",
        "zst",
        "rpm",
        "taz",
        "lzo",
        "jar",
        "tzo",
        "wad",
        "lz4",
        "gz",
        "arj",
        "Z",
        "pack",
        "zip",
        "bcpio",
        "lhz",
        "wkdownload",
        "crdownload",
        "part",
        "PAR2",
        "par2",
        "lrz",
        "pak",
        "cab",
        "theme",
        "rar",
        "sis",
        "tbz2",
        "tbz",
        "tb2",
        "7z",
        "deb",
        "udeb",
        "qp",
        "lz",
        "alz",
        "tlz",
        "lha",
        "lzh",
        "sit",
        "bz2",
        "bz",
        "flatpak",
        "xdgapp",
        "ustar",
        "lzma",
        "tgz",
        "xz",
        "dar",
        "zoo",
        "ace",
        "tar",
        "gtar",
        "gem",
        "sv4crc",
        "themepack",
        "tzst",
        "a",
        "ar",
        "txz",
        "xar",
        "pkg",
        "cpio"
    )
    private val presentationFormats = listOf(
        // extensions below this comment are extracted from application mime types
        "sldx",
        "kpr",
        "kpt",
        "ppsx",
        "otp",
        "otp",
        "potx",
        "potx",
        "sldm",
        "potm",
        "potm",
        "key",
        "ppam",
        "odp",
        "odp",
        "sdd",
        "sdp",
        "fodp",
        "fodp",
        "sxi",
        "sxi",
        "ppz",
        "ppt",
        "pps",
        "pot",
        "ppt",
        "pptx",
        "pptx",
        "pptm",
        "pptm",
        "ppsm",
        "sti",
        "sti",
        "mgp"
    )

    fun getDrawableFromFTPFile(file: File): Int {
        return if (file.isDirectory) {
            R.drawable.folder
        } else if (file.isFile && getFileFormat(file) in audioFormats) {
            R.drawable.music_note
        } else if (file.isFile && getFileFormat(file) in videoFormats) {
            R.drawable.local_movies
        } else if (file.isFile && (isImage(file.name))) {
            R.drawable.image
        } else if (file.isFile && getFileFormat(file) in fontFormats) {
            R.drawable.font_download
        } else if (file.isFile && getFileFormat(file) in spreadsheetFormats) {
            R.drawable.table_chart
        } else if (file.isFile && getFileFormat(file) in locationFormats) {
            R.drawable.location_on
        } else if (file.isFile && (getFileFormat(file) in documentFormats || getFileFormat(file) in plainTextFormats)) {
            R.drawable.text_snippet
        } else if (file.isFile && getFileFormat(file) in packageFormats) {
            R.drawable.archive
        } else if (file.isFile && getFileFormat(file) in presentationFormats) {
            R.drawable.slideshow
        } else if (file.isFile) {
            R.drawable.insert_drive_file
        } else if (file.isSymbolicLink) {
            R.drawable.link
        } else {
            R.drawable.error
        }
    }

    private fun getFileFormat(file: File): String {
        return getFileFormat(file.name)
    }

    private fun getFileFormat(filename: String): String {
        val list = filename.split(".")
        return if (list.size < 2) {
            ""
        } else {
            list.last().lowercase()
        }
    }

    fun isImage(filename: String): Boolean {
        return getFileFormat(filename) in imageFormats || getFileFormat(filename) in previewableImageFormats
    }

    fun previewable(filename: String): Boolean {
        return getFileFormat(filename) in previewableImageFormats || getFileFormat(filename) in plainTextFormats
    }
}
